
 var fs      = require('fs');
 var mime    = require('mime');
 var sys     = require('sys');
 var url     = require('url');
 var zlib    = require('zlib');
 var console = require('console-plus');
 var Service = require('./service.js');

 function readFile (sourceWWW, pathWWW, stats, req, res, opt){
     var readStream = fs.createReadStream(sourceWWW);
     var acceptEncoding = req.headers['accept-encoding'];

     if (!acceptEncoding) {
         acceptEncoding = '';
     }

     res.setHeader('Content-Type', mime.lookup(sourceWWW)); //Transfer-Encoding: chunked
     //res.setHeader('Transfer-Encoding', 'chunked');

     readStream.on('open', function () {
         if ((opt && opt.GZIP) && acceptEncoding.match(/\bgzip\b/)) {
             res.setHeader('content-encoding', 'gzip');

             readStream.pipe(zlib.createGzip()).pipe(res);
         } else if ((opt && opt.GZIP) && acceptEncoding.match(/\bdeflate\b/)) {
             res.setHeader('content-encoding', 'deflate');

             readStream.pipe(zlib.createDeflate()).pipe(res);
         } else {
             readStream.pipe(res);
         }
     });

     readStream.on("end", function(){
         console.log("End of file read" + sourceWWW, console.logLevel.L_Full);
     });

     readStream.on('error', function() {
         console.err("Can't load path: " + sourceWWW);

         process.nextTick(function(){
             res.writeHead(301, {
                 'Location': pathWWW + '/',
                 'Expires' : (new Date).toGMTString()
             });

             res.end("");
         })
     });

     res.on('close', function(){
         readStream.destroy();
     })
 }

 function handle (sourceWWW, pathWWW, req, res, opt){
 	if (req.method !== 'GET') {
		res.writeHead(404, {
			'Content-Type': 'text/plain'
		});
		
		res.end('invalid method:' + req.method);
	} else {
        console.log('Load file: ' + sourceWWW, console.logLevel.L_Full);

        sourceWWW = sourceWWW.replace(/\\/g, '/');
        pathWWW   = pathWWW.replace(/\\/g, '/');

        if (Service.ExtractFileName(sourceWWW) == '') {
            if (sourceWWW[sourceWWW.length - 1] != "/") sourceWWW += "/";

            if (pathWWW != "/") sourceWWW += pathWWW;

            if (sourceWWW[sourceWWW.length - 1] == "/"){
                sourceWWW += 'index.html';
            }
        }

        fs.stat(sourceWWW, function (err, stats){
            if (err){
                var _ip = req.connection.remoteAddress;

                res.writeHead(404, {'Content-Type' : 'text/plain'});
                res.end('Error 404 :-(\n\nPath: "' + pathWWW + '" not found!');

                console.err(_ip + ' File: "' + sourceWWW + '" not found (on "stat" cmd). ' + err);
            } else {
                readFile(sourceWWW, pathWWW, stats, req, res, opt)
            }
        })
    }
 }

 exports.handle = handle;